@@ -17,52 +17,76 @@ fn derive_container_set_by_index_impl(
     generics: &Generics,
 ) -> TokenStream {
     match data {
-        Data::Struct(ref data) => match data.fields {
-            Fields::Named(ref fields) => {
-                let set_by_field = fields.named.iter().enumerate().map(|(i, f)| {
-                    let field_name = &f.ident;
-                    let field_type = &f.ty;
-                    quote_spanned! { f.span() =>
-                                     #i => {
-                                         let result = <#field_type>::deserialize(encoding)?;
-                                         self.#field_name = result;
-                                     },
-                    }
-                });
+        Data::Struct(ref data) => {
+            let fields = match data.fields {
+                // "regular" struct with 1+ fields
+                Fields::Named(ref fields) => &fields.named,
+                // "tuple" struct
+                // only support the case with one unnamed field, to support "newtype" pattern
+                Fields::Unnamed(ref fields) => &fields.unnamed,
+                _ => unimplemented!(
+                    "this type of struct is currently not supported by this derive macro"
+                ),
+            };
 
-                let impl_impl = if generics.params.is_empty() {
-                    quote! { #name }
-                } else {
-                    let (_, ty_generics, _) = generics.split_for_impl();
-                    quote! { #generics #name #ty_generics }
-                };
-                quote! {
-                    impl #impl_impl {
-                        fn __ssz_rs_set_by_index(&mut self, index: usize, encoding: &[u8]) -> Result<(), ssz_rs::DeserializeError> {
-                            match index {
-                                #(#set_by_field)*
-                                _ => unreachable!(),
-                            }
-                            Ok(())
+            let set_by_field = fields.iter().enumerate().map(|(i, f)| {
+                let field_type = &f.ty;
+                match &f.ident {
+                    Some(field_name) => quote_spanned! { f.span() =>
+                        #i => {
+                            let result = <#field_type>::deserialize(encoding)?;
+                            self.#field_name = result;
+                        },
+                    },
+                    None => quote_spanned! { f.span() =>
+                        #i => {
+                            let result = <#field_type>::deserialize(encoding)?;
+                            self.0 = result;
+                        },
+                    },
+                }
+            });
+
+            let impl_impl = if generics.params.is_empty() {
+                quote! { #name }
+            } else {
+                let (_, ty_generics, _) = generics.split_for_impl();
+                quote! { #generics #name #ty_generics }
+            };
+            quote! {
+                impl #impl_impl {
+                    fn __ssz_rs_set_by_index(&mut self, index: usize, encoding: &[u8]) -> Result<(), ssz_rs::DeserializeError> {
+                        match index {
+                            #(#set_by_field)*
+                            _ => unreachable!(),
                         }
+                        Ok(())
                     }
                 }
             }
-            _ => unreachable!(),
-        },
+        }
         Data::Enum(..) => quote! {},
-        Data::Union(..) => unreachable!(),
+        Data::Union(..) => unreachable!("data was already validated to exclude union types"),
     }
 }
 
 fn derive_serialize_impl(data: &Data) -> TokenStream {
     match data {
-        Data::Struct(ref data) => match data.fields {
-            Fields::Named(ref fields) => {
-                let serialization_by_field = fields.named.iter().map(|f| {
-                    let field_name = &f.ident;
-                    let field_type = &f.ty;
-                    quote_spanned! { f.span() =>
+        Data::Struct(ref data) => {
+            let fields = match data.fields {
+                // "regular" struct with 1+ fields
+                Fields::Named(ref fields) => &fields.named,
+                // "tuple" struct
+                // only support the case with one unnamed field, to support "newtype" pattern
+                Fields::Unnamed(ref fields) => &fields.unnamed,
+                _ => unimplemented!(
+                    "this type of struct is currently not supported by this derive macro"
+                ),
+            };
+            let serialization_by_field = fields.iter().map(|f| {
+                let field_type = &f.ty;
+                match &f.ident {
+                    Some(field_name) => quote_spanned! { f.span() =>
                         let mut element_buffer = Vec::with_capacity(<#field_type>::size_hint());
                         self.#field_name.serialize(&mut element_buffer)?;
 
@@ -77,24 +101,39 @@ fn derive_serialize_impl(data: &Data) -> TokenStream {
                             fixed_lengths_sum += buffer_len;
                             variable_lengths.push(0)
                         }
-                    }
-                });
+                    },
+                    None => quote_spanned! { f.span() =>
+                        let mut element_buffer = Vec::with_capacity(<#field_type>::size_hint());
+                        self.0.serialize(&mut element_buffer)?;
 
-                quote! {
-                    fn serialize(&self, buffer: &mut Vec<u8>) -> Result<usize, ssz_rs::SerializeError> {
-                        let mut fixed = vec![];
-                        let mut variable = vec![];
-                        let mut variable_lengths = vec![];
-                        let mut fixed_lengths_sum = 0;
+                        let buffer_len = element_buffer.len();
+                        if <#field_type>::is_variable_size() {
+                            fixed.push(None);
+                            fixed_lengths_sum += #BYTES_PER_LENGTH_OFFSET;
+                            variable.push(element_buffer);
+                            variable_lengths.push(buffer_len);
+                        } else {
+                            fixed.push(Some(element_buffer));
+                            fixed_lengths_sum += buffer_len;
+                            variable_lengths.push(0)
+                        }
+                    },
+                }
+            });
 
-                        #(#serialization_by_field)*
+            quote! {
+                fn serialize(&self, buffer: &mut Vec<u8>) -> Result<usize, ssz_rs::SerializeError> {
+                    let mut fixed = vec![];
+                    let mut variable = vec![];
+                    let mut variable_lengths = vec![];
+                    let mut fixed_lengths_sum = 0;
 
-                        ssz_rs::internal::serialize_composite_from_components(fixed, variable, variable_lengths, fixed_lengths_sum, buffer)
-                    }
+                    #(#serialization_by_field)*
+
+                    ssz_rs::internal::serialize_composite_from_components(fixed, variable, variable_lengths, fixed_lengths_sum, buffer)
                 }
             }
-            _ => unreachable!(),
-        },
+        }
         Data::Enum(ref data) => {
             let serialization_by_variant = data.variants.iter().enumerate().map(|(i, variant)| {
                 let variant_name = &variant.ident;
@@ -128,18 +167,27 @@ fn derive_serialize_impl(data: &Data) -> TokenStream {
                 }
             }
         }
-        Data::Union(..) => unreachable!(),
+        Data::Union(..) => unreachable!("data was already validated to exclude union types"),
     }
 }
 
 fn derive_deserialize_impl(data: &Data) -> TokenStream {
     match data {
-        Data::Struct(ref data) => match data.fields {
-            Fields::Named(ref fields) => {
-                let deserialization_by_field = fields.named.iter().enumerate().map(|(i, f)| {
-                    let field_name = &f.ident;
-                    let field_type = &f.ty;
-                    quote_spanned! { f.span() =>
+        Data::Struct(ref data) => {
+            let fields = match data.fields {
+                // "regular" struct with 1+ fields
+                Fields::Named(ref fields) => &fields.named,
+                // "tuple" struct
+                // only support the case with one unnamed field, to support "newtype" pattern
+                Fields::Unnamed(ref fields) => &fields.unnamed,
+                _ => unimplemented!(
+                    "this type of struct is currently not supported by this derive macro"
+                ),
+            };
+            let deserialization_by_field = fields.iter().enumerate().map(|(i, f)| {
+                let field_type = &f.ty;
+                match &f.ident {
+                    Some(field_name) => quote_spanned! { f.span() =>
                         let bytes_read = if <#field_type>::is_variable_size() {
                             let end = start + #BYTES_PER_LENGTH_OFFSET;
                             let next_offset = u32::deserialize(&encoding[start..end])?;
@@ -154,45 +202,60 @@ fn derive_deserialize_impl(data: &Data) -> TokenStream {
                             encoded_length
                         };
                         start += bytes_read;
+                    },
+                    None => quote_spanned! { f.span() =>
+                        let bytes_read = if <#field_type>::is_variable_size() {
+                            let end = start + #BYTES_PER_LENGTH_OFFSET;
+                            let next_offset = u32::deserialize(&encoding[start..end])?;
+                            offsets.push((#i, next_offset as usize));
+
+                            #BYTES_PER_LENGTH_OFFSET
+                        } else {
+                            let encoded_length = <#field_type>::size_hint();
+                            let end = start + encoded_length;
+                            let result = <#field_type>::deserialize(&encoding[start..end])?;
+                            container.0 = result;
+                            encoded_length
+                        };
+                        start += bytes_read;
+                    },
+                }
+            });
+
+            quote! {
+                fn deserialize(encoding: &[u8]) -> Result<Self, ssz_rs::DeserializeError> {
+                    let mut start = 0;
+                    let mut offsets = vec![];
+                    let mut container = Self::default();
+
+                    #(#deserialization_by_field)*
+
+                    let mut total_bytes_read = start;
+
+                    // NOTE: this value is not read
+                    let dummy_index = 0;
+                    offsets.push((dummy_index, encoding.len()));
+
+                    for span in offsets.windows(2) {
+                        let (index, start) = span[0];
+                        let (_, end) = span[1];
+
+                        container.__ssz_rs_set_by_index(index, &encoding[start..end])?;
+                        total_bytes_read += end - start;
                     }
-                });
 
-                quote! {
-                    fn deserialize(encoding: &[u8]) -> Result<Self, ssz_rs::DeserializeError> {
-                        let mut start = 0;
-                        let mut offsets = vec![];
-                        let mut container = Self::default();
-
-                        #(#deserialization_by_field)*
-
-                        let mut total_bytes_read = start;
-
-                        // NOTE: this value is not read
-                        let dummy_index = 0;
-                        offsets.push((dummy_index, encoding.len()));
-
-                        for span in offsets.windows(2) {
-                            let (index, start) = span[0];
-                            let (_, end) = span[1];
-
-                            container.__ssz_rs_set_by_index(index, &encoding[start..end])?;
-                            total_bytes_read += end - start;
-                        }
-
-                        if total_bytes_read > encoding.len() {
-                            return Err(ssz_rs::DeserializeError::InputTooShort);
-                        }
-
-                        if total_bytes_read < encoding.len() {
-                            return Err(ssz_rs::DeserializeError::ExtraInput);
-                        }
-
-                        Ok(container)
+                    if total_bytes_read > encoding.len() {
+                        return Err(ssz_rs::DeserializeError::InputTooShort);
                     }
+
+                    if total_bytes_read < encoding.len() {
+                        return Err(ssz_rs::DeserializeError::ExtraInput);
+                    }
+
+                    Ok(container)
                 }
             }
-            _ => unreachable!(),
-        },
+        }
         Data::Enum(ref data) => {
             let deserialization_by_variant =
                 data.variants.iter().enumerate().map(|(i, variant)| {
@@ -229,59 +292,135 @@ fn derive_deserialize_impl(data: &Data) -> TokenStream {
                 }
             }
         }
-        Data::Union(..) => unreachable!(),
+        Data::Union(..) => unreachable!("data was already validated to exclude union types"),
     }
 }
 
 fn derive_variable_size_impl(data: &Data) -> TokenStream {
     match data {
-        Data::Struct(ref data) => match data.fields {
-            Fields::Named(ref fields) => {
-                let impl_by_field = fields.named.iter().map(|f| {
-                    let field_type = &f.ty;
-                    quote_spanned! { f.span() =>
-                        <#field_type>::is_variable_size()
-                    }
-                });
-
-                quote! {
-                    #(#impl_by_field)|| *
+        Data::Struct(ref data) => {
+            let fields = match data.fields {
+                Fields::Named(ref fields) => &fields.named,
+                Fields::Unnamed(ref fields) => &fields.unnamed,
+                _ => unimplemented!(
+                    "this type of struct is currently not supported by this derive macro"
+                ),
+            };
+            let impl_by_field = fields.iter().map(|f| {
+                let field_type = &f.ty;
+                quote_spanned! { f.span() =>
+                    <#field_type>::is_variable_size()
                 }
+            });
+
+            quote! {
+                #(#impl_by_field)|| *
             }
-            _ => unreachable!(),
-        },
+        }
         Data::Enum(..) => {
             quote! { true }
         }
-        Data::Union(..) => unreachable!(),
+        Data::Union(..) => unreachable!("data was already validated to exclude union types"),
     }
 }
 
 fn derive_size_hint_impl(data: &Data) -> TokenStream {
     match data {
-        Data::Struct(ref data) => match data.fields {
-            Fields::Named(ref fields) => {
-                let impl_by_field = fields.named.iter().map(|f| {
-                    let field_type = &f.ty;
-                    quote_spanned! { f.span() =>
-                        <#field_type>::size_hint()
-                    }
-                });
+        Data::Struct(ref data) => {
+            let fields = match data.fields {
+                Fields::Named(ref fields) => &fields.named,
+                Fields::Unnamed(ref fields) => &fields.unnamed,
+                _ => unimplemented!(
+                    "this type of struct is currently not supported by this derive macro"
+                ),
+            };
+            let impl_by_field = fields.iter().map(|f| {
+                let field_type = &f.ty;
+                quote_spanned! { f.span() =>
+                    <#field_type>::size_hint()
+                }
+            });
 
-                quote! {
-                    if Self::is_variable_size() {
-                        0
-                    } else {
-                        #(#impl_by_field)+ *
-                    }
+            quote! {
+                if Self::is_variable_size() {
+                    0
+                } else {
+                    #(#impl_by_field)+ *
                 }
             }
-            _ => unreachable!(),
-        },
+        }
         Data::Enum(..) => {
             quote! { 0 }
         }
-        Data::Union(..) => unreachable!(),
+        Data::Union(..) => unreachable!("data was already validated to exclude union types"),
+    }
+}
+
+fn derive_merkleization_impl(data: &Data) -> TokenStream {
+    match data {
+        Data::Struct(ref data) => {
+            let fields = match data.fields {
+                Fields::Named(ref fields) => &fields.named,
+                Fields::Unnamed(ref fields) => &fields.unnamed,
+                _ => unimplemented!(
+                    "this type of struct is currently not supported by this derive macro"
+                ),
+            };
+            let field_count = fields.iter().len();
+            let impl_by_field = fields.iter().enumerate().map(|(i, f)| match &f.ident {
+                Some(field_name) => quote_spanned! { f.span() =>
+                    let chunk = self.#field_name.hash_tree_root(context)?;
+                    let range = #i*#BYTES_PER_CHUNK..(#i+1)*#BYTES_PER_CHUNK;
+                    chunks[range].copy_from_slice(chunk.as_ref());
+                },
+                None => quote_spanned! { f.span() =>
+                    let chunk = self.0.hash_tree_root(context)?;
+                    let range = #i*#BYTES_PER_CHUNK..(#i+1)*#BYTES_PER_CHUNK;
+                    chunks[range].copy_from_slice(chunk.as_ref());
+                },
+            });
+            quote! {
+                fn hash_tree_root(&self, context: &ssz_rs::MerkleizationContext) -> Result<ssz_rs::Root, ssz_rs::MerkleizationError> {
+                    let mut chunks = vec![0u8; #field_count * #BYTES_PER_CHUNK];
+                    #(#impl_by_field)*
+                    ssz_rs::internal::merkleize(&chunks, None, context)
+                }
+            }
+        }
+        Data::Enum(ref data) => {
+            let hash_tree_root_by_variant = data.variants.iter().enumerate().map(|(i, variant)| {
+                let variant_name = &variant.ident;
+                match &variant.fields {
+                    Fields::Unnamed(..) => {
+                        quote_spanned! { variant.span() =>
+                            Self::#variant_name(value) => {
+                                let selector = #i as u8 as usize;
+                                let data_root  = value.hash_tree_root(context)?;
+                                Ok(ssz_rs::internal::mix_in_selector(&data_root, selector, context))
+                            }
+                        }
+                    }
+                    Fields::Unit => {
+                        quote_spanned! { variant.span() =>
+                            Self::None => Ok(ssz_rs::internal::mix_in_selector(
+                                &ssz_rs::Root::default(),
+                                0,
+                                context,
+                            )),
+                        }
+                    }
+                    _ => unreachable!(),
+                }
+            });
+            quote! {
+                fn hash_tree_root(&self, context: &ssz_rs::MerkleizationContext) -> Result<ssz_rs::Root, ssz_rs::MerkleizationError> {
+                    match self {
+                            #(#hash_tree_root_by_variant)*
+                    }
+                }
+            }
+        }
+        Data::Union(..) => unreachable!("data was already validated to exclude union types"),
     }
 }
 
@@ -289,6 +428,12 @@ fn is_valid_none_identifier(ident: &Ident) -> bool {
     *ident == format_ident!("None")
 }
 
+// Refers to the validation state of proc macro's input
+enum ValidationState<'a> {
+    Unvalidated(&'a Data),
+    Validated(&'a Data),
+}
+
 // Validates the incoming data follows the rules
 // for mapping the Rust term to something that can
 // implement the `SimpleSerialize` trait.
@@ -305,7 +450,8 @@ fn validate_derive_data(data: ValidationState) -> ValidationState {
                     panic!("ssz_rs containers with no fields are illegal")
                 }
             }
-            _ => panic!("Structs with unit or unnnamed fields are not supported"),
+            Fields::Unnamed(ref fields) if fields.unnamed.len() == 1 => {}
+            _ => panic!("Structs with unit or multiple unnnamed fields are not supported"),
         },
         Data::Enum(ref data) => {
             if data.variants.is_empty() {
@@ -358,77 +504,10 @@ fn validate_derive_data(data: ValidationState) -> ValidationState {
     ValidationState::Validated(data)
 }
 
-fn derive_merkleization_impl(data: &Data) -> TokenStream {
-    match data {
-        Data::Struct(ref data) => match data.fields {
-            Fields::Named(ref fields) => {
-                let field_count = fields.named.iter().len();
-                let impl_by_field = fields.named.iter().enumerate().map(|(i, f)| {
-                    let field_name = &f.ident;
-                    quote_spanned! { f.span() =>
-                        let chunk = self.#field_name.hash_tree_root(context)?;
-                        let range = #i*#BYTES_PER_CHUNK..(#i+1)*#BYTES_PER_CHUNK;
-                        chunks[range].copy_from_slice(chunk.as_ref());
-                    }
-                });
-                quote! {
-                    fn hash_tree_root(&self, context: &ssz_rs::MerkleizationContext) -> Result<ssz_rs::Root, ssz_rs::MerkleizationError> {
-                        let mut chunks = vec![0u8; #field_count * #BYTES_PER_CHUNK];
-                        #(#impl_by_field)*
-                        ssz_rs::internal::merkleize(&chunks, None, context)
-                    }
-                }
-            }
-            _ => unreachable!(),
-        },
-        Data::Enum(ref data) => {
-            let hash_tree_root_by_variant = data.variants.iter().enumerate().map(|(i, variant)| {
-                let variant_name = &variant.ident;
-                match &variant.fields {
-                    Fields::Unnamed(..) => {
-                        quote_spanned! { variant.span() =>
-                            Self::#variant_name(value) => {
-                                let selector = #i as u8 as usize;
-                                let data_root  = value.hash_tree_root(context)?;
-                                Ok(ssz_rs::internal::mix_in_selector(&data_root, selector, context))
-                            }
-                        }
-                    }
-                    Fields::Unit => {
-                        quote_spanned! { variant.span() =>
-                            Self::None => Ok(ssz_rs::internal::mix_in_selector(
-                                &ssz_rs::Root::default(),
-                                0,
-                                context,
-                            )),
-                        }
-                    }
-                    _ => unreachable!(),
-                }
-            });
-            quote! {
-                fn hash_tree_root(&self, context: &ssz_rs::MerkleizationContext) -> Result<ssz_rs::Root, ssz_rs::MerkleizationError> {
-                    match self {
-                            #(#hash_tree_root_by_variant)*
-                    }
-                }
-            }
-        }
-        Data::Union(..) => unreachable!(),
-    }
-}
-
-// Refers to the validation state of proc macro's input
-enum ValidationState<'a> {
-    Unvalidated(&'a Data),
-    Validated(&'a Data),
-}
-
 #[proc_macro_derive(SimpleSerialize)]
 pub fn derive(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
     let input = parse_macro_input!(input as DeriveInput);
 
-    let name = &input.ident;
     let data = ValidationState::Unvalidated(&input.data);
 
     let data = &validate_derive_data(data);
@@ -438,6 +517,7 @@ pub fn derive(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
         ValidationState::Unvalidated(..) => panic!("do not process unvalidated input"),
     };
 
+    let name = &input.ident;
     let generics = &input.generics;
     let set_by_index_impl = derive_container_set_by_index_impl(name, data, generics);
     let serialize_impl = derive_serialize_impl(data);
